''' This utility module allow to instantiate object from a given package/path '''

import importlib
import pkgutil
import logging

from .caseinsensitivedict import CaseInsensitiveDict

CLASS_NAME_ATTR = '__dynamoclass__'

_logger = logging.getLogger(__name__)
_dynamo_cache = CaseInsensitiveDict()

def load_classes(package_str):
    '''Load all classes from modules of a given `package_str`. All class instances are stored in a case-insensitive `dict`
    and returned. If a package doesn't contain any class `None` is returned'''

    _logger.debug('Loading all modules from %s', package_str)
    package = importlib.import_module(package_str)
    package_path = package.__path__
    _logger.debug('Searching for modules in package %s (%s)', package_str, package_path)
    for _, name, ispkg in pkgutil.iter_modules(package_path, package_str + "."):
        if not ispkg:
            _logger.debug('Found module: %s', name)
            module = importlib.import_module(name)
            if hasattr(module, CLASS_NAME_ATTR):
                class_name = getattr(module, CLASS_NAME_ATTR)
                _logger.debug('Found class: %s', class_name)
                clasz = getattr(module, class_name)

                if package_str not in _dynamo_cache:
                    _dynamo_cache[package_str] = CaseInsensitiveDict()
                
                if class_name not in _dynamo_cache[package_str]:
                    _dynamo_cache[package_str][class_name] = clasz
                    _logger.debug('Correctly loaded class: %s from: "%s"', class_name, package_str)
                else:
                    _logger.warning('Already loaded class: %s from: "%s"', class_name, package_str)
            else:
                _logger.warning('Module inside %s does not contain required attribute: %s', package_str, CLASS_NAME_ATTR)
        else:
            _logger.warning('Ignoring package: %s', name)

    if package_str in _dynamo_cache:
        return _dynamo_cache[package_str]
    else:
        return None


def get_all(package_str):
    '''Retrieve all classes of a given package. All arguments are case-insensitive''' 
    if (package_str in _dynamo_cache):
        return _dynamo_cache[package_str]
    return None

def get(package_str, classname):
    '''Retrieve from the internal cache a class instance. All arguments are case-insensitive''' 
    if (package_str in _dynamo_cache) and (classname in _dynamo_cache[package_str]):
        return _dynamo_cache[package_str][classname]
    return None
        
        


"""
    From an idea of https://gist.github.com/nguyenkims hosted on: https://gist.github.com/nguyenkims/e92df0f8bd49973f0c94bddf36ed7fd0
"""
import logging
import sys
from logging.handlers import  RotatingFileHandler

FORMATTER = None
LOG_FILE = None
LEVEL = logging.DEBUG
PROPAGATE = True

# RotatingFileHandler properties
MAX_BYTES = 5000000 # 5MB
BACKUP_COUNT = 5

_loggers_cache = {}

def _get_console_handler():
	console_handler = logging.StreamHandler(sys.stdout)
	console_handler.setFormatter(FORMATTER)
	return console_handler

def _get_file_handler():
	file_handler = RotatingFileHandler(LOG_FILE, backupCount=BACKUP_COUNT, maxBytes=MAX_BYTES)
	file_handler.setFormatter(FORMATTER)
	return file_handler

def get_logger(logger_name) -> logging.Logger:
    if logger_name not in _loggers_cache:
        logger = logging.getLogger(logger_name)

        logger.setLevel(LEVEL) # better to have too much log than not enough

        logger.addHandler(_get_console_handler())

        if LOG_FILE is not None:
            logger.addHandler(_get_file_handler())

        # with this pattern, it'zs rarely necessary to propagate the error up to parent
        logger.propagate = PROPAGATE

        #
        _loggers_cache[logger_name] = logger
    else:
        logger = _loggers_cache[logger_name]
    
    return logger


class CaseInsensitiveDict(dict):
    def __setitem__(self, key, value):
        super(CaseInsensitiveDict, self).__setitem__(key.lower(), value)

    def __getitem__(self, key):
        return super(CaseInsensitiveDict, self).__getitem__(key.lower())

    def __contains__(self, key):
        return super(CaseInsensitiveDict, self).__contains__(key.lower())


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pycommon_andreacioni",
    version="0.0.3",
    author="Andrea Cioni",
    description="Common utility classes from some Python projects",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/andreacioni/pycommon",
    packages=setuptools.find_packages(),
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ),
)
